"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-bitwise */
const fs = require("fs");
const ts = require("typescript");
const IndentedWriter_1 = require("../IndentedWriter");
const TypeScriptHelpers_1 = require("../TypeScriptHelpers");
const Span_1 = require("./Span");
/**
 * An "Entry" is a type definition that we encounter while traversing the
 * references from the package entry point.  This data structure helps filter,
 * sort, and rename the entries that end up in the output package typings file.
 */
class Entry {
    constructor(parameters) {
        /**
         * If true, this entry should be emitted using the "export" keyword instead of the "declare" keyword.
         */
        this.exported = false;
        this._uniqueName = undefined;
        this._sortKey = undefined;
        this.localName = parameters.localName;
        this.followedSymbol = parameters.followedSymbol;
        this.importPackagePath = parameters.importPackagePath;
        this.importPackageExportName = parameters.importPackageExportName;
    }
    /**
     * The localName, possibly renamed to ensure that all the top-level exports have unique names.
     */
    get uniqueName() {
        return this._uniqueName;
    }
    set uniqueName(value) {
        this._uniqueName = value;
        this._sortKey = undefined; // invalidate the cached value
    }
    getSortKey() {
        if (!this._sortKey) {
            const name = this.uniqueName || this.localName;
            if (name.substr(0, 1) === '_') {
                // Removes the leading underscore, for example: "_example" --> "example*"
                // This causes internal definitions to sort alphabetically with regular definitions.
                // The star is appended to preserve uniqueness, since "*" is not a legal  identifier character.
                this._sortKey = name.substr(1) + '*';
            }
            else {
                this._sortKey = name;
            }
        }
        return this._sortKey;
    }
}
class PackageTypingsGenerator {
    constructor(context) {
        this._indentedWriter = new IndentedWriter_1.default();
        /**
         * A cache that tells us the Entry that is tracking a given symbol.  Because of aliases,
         * two different symbols can map to the same Entry object.
         */
        this._entriesBySymbol = new Map();
        /**
         * This data structure stores the same entries as _entriesBySymbol.values().
         * They are sorted according to Entry.getSortKey().
         */
        this._entries = [];
        this._context = context;
        this._typeChecker = context.typeChecker;
    }
    /**
     * Returns an ancestor of "node", such that the ancestor, any intermediary nodes,
     * and the starting node match a list of expected kinds.  Undefined is returned
     * if there aren't enough ancestors, or if the kinds are incorrect.
     *
     * For example, suppose child "C" has parents A --> B --> C.
     *
     * Calling _matchAncestor(C, [ExportSpecifier, NamedExports, ExportDeclaration])
     * would return A only if A is of kind ExportSpecifier, B is of kind NamedExports,
     * and C is of kind ExportDeclaration.
     *
     * Calling _matchAncestor(C, [ExportDeclaration]) would return C.
     */
    static _matchAncestor(node, kindsToMatch) {
        // (slice(0) clones an array)
        const reversedParentKinds = kindsToMatch.slice(0).reverse();
        let current = undefined;
        for (const parentKind of reversedParentKinds) {
            if (!current) {
                // The first time through, start with node
                current = node;
            }
            else {
                // Then walk the parents
                current = current.parent;
            }
            // If we ran out of items, or if the kind doesn't match, then fail
            if (!current || current.kind !== parentKind) {
                return undefined;
            }
        }
        // If we matched everything, then return the node that matched the last parentKinds item
        return current;
    }
    /**
     * Does a depth-first search of the children of the specified node.  Returns the first child
     * with the specified kind, or undefined if there is no match.
     */
    static _findFirstChildNode(node, kindToMatch) {
        for (const child of node.getChildren()) {
            if (child.kind === kindToMatch) {
                return child;
            }
            const recursiveMatch = this._findFirstChildNode(child, kindToMatch);
            if (recursiveMatch) {
                return recursiveMatch;
            }
        }
        return undefined;
    }
    /**
     * For the given symbol, follow imports and type alias to find the symbol that represents
     * the original definition.
     */
    static _followAliases(symbol, typeChecker) {
        let current = symbol;
        // Is it ambient?  We will examine all of the declarations we encounter
        // to see if any of them contains the "export" keyword; if not, then it's ambient.
        let isAmbient = true;
        while (true) {
            for (const declaration of current.declarations || []) {
                // 1. Check for any signs that this is not an ambient definition
                if (declaration.kind === ts.SyntaxKind.ExportSpecifier
                    || declaration.kind === ts.SyntaxKind.ExportAssignment) {
                    isAmbient = false;
                }
                const modifiers = ts.getCombinedModifierFlags(declaration);
                if (modifiers & (ts.ModifierFlags.Export | ts.ModifierFlags.ExportDefault)) {
                    isAmbient = false;
                }
                // 2. Check for any signs that this was imported from an external package
                let result;
                result = PackageTypingsGenerator._followAliasesForExportDeclaration(declaration, current);
                if (result) {
                    return result;
                }
                result = PackageTypingsGenerator._followAliasesForImportDeclaration(declaration, current);
                if (result) {
                    return result;
                }
            }
            if (!(current.flags & ts.SymbolFlags.Alias)) {
                break;
            }
            const currentAlias = TypeScriptHelpers_1.default.getImmediateAliasedSymbol(current, typeChecker);
            // Stop if we reach the end of the chain
            if (!currentAlias || currentAlias === current) {
                break;
            }
            current = currentAlias;
        }
        return {
            symbol: current,
            importPackagePath: undefined,
            importPackageExportName: undefined,
            isAmbient: isAmbient
        };
    }
    /**
     * Helper function for _followAliases(), for handling ts.ExportDeclaration patterns
     */
    static _followAliasesForExportDeclaration(declaration, symbol) {
        // EXAMPLE:
        // "export { A } from './file-a';"
        //
        // ExportDeclaration:
        //   ExportKeyword:  pre=[export] sep=[ ]
        //   NamedExports:
        //     FirstPunctuation:  pre=[{] sep=[ ]
        //     SyntaxList:
        //       ExportSpecifier:  <------------- declaration
        //         Identifier:  pre=[A] sep=[ ]
        //     CloseBraceToken:  pre=[}] sep=[ ]
        //   FromKeyword:  pre=[from] sep=[ ]
        //   StringLiteral:  pre=['./file-a']
        //   SemicolonToken:  pre=[;]
        const exportDeclaration = PackageTypingsGenerator._matchAncestor(declaration, [ts.SyntaxKind.ExportDeclaration, ts.SyntaxKind.NamedExports, ts.SyntaxKind.ExportSpecifier]);
        if (exportDeclaration && exportDeclaration.moduleSpecifier) {
            // Examples:
            //    " '@microsoft/sp-lodash-subset'"
            //    " "lodash/has""
            const packagePath = PackageTypingsGenerator._getPackagePathFromModuleSpecifier(exportDeclaration.moduleSpecifier);
            if (packagePath) {
                // Example: " ExportName as RenamedName"
                const exportSpecifier = declaration;
                const importPackageExportName = (exportSpecifier.propertyName || exportSpecifier.name).getText();
                return {
                    symbol: symbol,
                    importPackagePath: packagePath,
                    importPackageExportName: importPackageExportName.trim(),
                    isAmbient: false
                };
            }
        }
        return undefined;
    }
    /**
     * Helper function for _followAliases(), for handling ts.ImportDeclaration patterns
     */
    static _followAliasesForImportDeclaration(declaration, symbol) {
        // EXAMPLE:
        // "import * as theLib from 'the-lib';"
        //
        // ImportDeclaration:
        //   ImportKeyword:  pre=[import] sep=[ ]
        //   ImportClause:
        //     NamespaceImport:  <------------- declaration
        //       AsteriskToken:  pre=[*] sep=[ ]
        //       AsKeyword:  pre=[as] sep=[ ]
        //       Identifier:  pre=[theLib] sep=[ ]
        //   FromKeyword:  pre=[from] sep=[ ]
        //   StringLiteral:  pre=['the-lib']
        //   SemicolonToken:  pre=[;]
        const importDeclaration = PackageTypingsGenerator._matchAncestor(declaration, [ts.SyntaxKind.ImportDeclaration, ts.SyntaxKind.ImportClause, ts.SyntaxKind.NamespaceImport]);
        if (importDeclaration && importDeclaration.moduleSpecifier) {
            // Examples:
            //    " '@microsoft/sp-lodash-subset'"
            //    " "lodash/has""
            const packagePath = PackageTypingsGenerator._getPackagePathFromModuleSpecifier(importDeclaration.moduleSpecifier);
            if (packagePath) {
                return {
                    symbol: symbol,
                    importPackagePath: packagePath,
                    importPackageExportName: '*',
                    isAmbient: false
                };
            }
        }
    }
    static _getPackagePathFromModuleSpecifier(moduleSpecifier) {
        // Examples:
        //    " '@microsoft/sp-lodash-subset'"
        //    " "lodash/has""
        //    " './MyClass'"
        const moduleSpecifierText = moduleSpecifier.getFullText();
        // Remove quotes/whitespace
        const path = moduleSpecifierText
            .replace(/^\s*['"]/, '')
            .replace(/['"]\s*$/, '');
        // Does it start with something like "./"?
        // If not, then assume it's an import from an external package
        if (!/^\.\//.test(path)) {
            return path;
        }
        return undefined;
    }
    /**
     * Generates the typings file and writes it to disk.
     *
     * @param dtsFilename    - The *.d.ts output filename
     */
    writeTypingsFile(dtsFilename) {
        const fileContent = this.generateTypingsFileContent();
        fs.writeFileSync(dtsFilename, fileContent);
    }
    generateTypingsFileContent() {
        this._indentedWriter.spacing = '';
        this._indentedWriter.clear();
        const packageSymbol = this._context.package.getDeclarationSymbol();
        const exportSymbols = this._typeChecker.getExportsOfModule(packageSymbol) || [];
        for (const exportSymbol of exportSymbols) {
            const entry = this._fetchEntryForSymbol(exportSymbol);
            if (!entry) {
                // This is an export of the current package, but for some reason _fetchEntryForSymbol()
                // can't analyze it.
                this._indentedWriter.writeLine('// Unsupported re-export: ' + exportSymbol.name);
            }
            else {
                entry.exported = true;
            }
        }
        this._makeUniqueNames();
        this._entries.sort((a, b) => a.getSortKey().localeCompare(b.getSortKey()));
        // If there is a @packagedocumentation header, put it first:
        const packageDocumentation = this._context.package.documentation.originalAedoc;
        if (packageDocumentation) {
            this._indentedWriter.writeLine(TypeScriptHelpers_1.default.formatJSDocContent(packageDocumentation));
            this._indentedWriter.writeLine();
        }
        // Emit the imports first
        for (const entry of this._entries) {
            if (entry.importPackagePath) {
                if (entry.importPackageExportName === '*') {
                    this._indentedWriter.write(`import * as ${entry.uniqueName}`);
                }
                else if (entry.uniqueName !== entry.importPackageExportName) {
                    this._indentedWriter.write(`import { ${entry.importPackageExportName} as ${entry.uniqueName} }`);
                }
                else {
                    this._indentedWriter.write(`import { ${entry.importPackageExportName} }`);
                }
                this._indentedWriter.writeLine(` from '${entry.importPackagePath}';`);
            }
        }
        // Emit the regular declarations
        for (const entry of this._entries) {
            if (!entry.importPackagePath) {
                // If it's local, then emit all the declarations
                for (const declaration of entry.followedSymbol.declarations || []) {
                    const span = new Span_1.Span(declaration);
                    this._modifySpan(span, entry);
                    this._indentedWriter.writeLine();
                    this._indentedWriter.writeLine(span.getModifiedText());
                }
            }
        }
        // Normalize to CRLF
        const fileContent = this._indentedWriter.toString().replace(/\r?\n/g, '\r\n');
        return fileContent;
    }
    /**
     * Before writing out a declaration, _modifySpan() applies various fixups to make it nice.
     */
    _modifySpan(rootSpan, entry) {
        rootSpan.modify((span, previousSpan, parentSpan) => {
            switch (span.kind) {
                case ts.SyntaxKind.ExportKeyword:
                case ts.SyntaxKind.DefaultKeyword:
                case ts.SyntaxKind.DeclareKeyword:
                    // Delete any explicit "export" or "declare" keywords -- we will re-add them below
                    span.modification.skipAll();
                    break;
                case ts.SyntaxKind.InterfaceKeyword:
                case ts.SyntaxKind.ClassKeyword:
                case ts.SyntaxKind.EnumKeyword:
                case ts.SyntaxKind.NamespaceKeyword:
                case ts.SyntaxKind.ModuleKeyword:
                case ts.SyntaxKind.TypeKeyword:
                case ts.SyntaxKind.FunctionKeyword:
                    // Replace the stuff we possibly deleted above
                    let replacedModifiers = 'declare ';
                    if (entry.exported) {
                        replacedModifiers = 'export ' + replacedModifiers;
                    }
                    if (previousSpan && previousSpan.kind === ts.SyntaxKind.SyntaxList) {
                        // If there is a previous span of type SyntaxList, then apply it before any other modifiers
                        // (e.g. "abstract") that appear there.
                        previousSpan.modification.prefix = replacedModifiers + previousSpan.modification.prefix;
                    }
                    else {
                        // Otherwise just stick it in front of this span
                        span.modification.prefix = replacedModifiers + span.modification.prefix;
                    }
                    break;
                case ts.SyntaxKind.VariableDeclaration:
                    if (!parentSpan) {
                        // The VariableDeclaration node is part of a VariableDeclarationList, however
                        // the Entry.followedSymbol points to the VariableDeclaration part because
                        // multiple definitions might share the same VariableDeclarationList.
                        //
                        // Since we are emitting a separate declaration for each one, we need to look upwards
                        // in the ts.Node tree and write a copy of the enclosing VariableDeclarationList
                        // content (e.g. "var" from "var x=1, y=2").
                        const list = PackageTypingsGenerator._matchAncestor(span.node, [ts.SyntaxKind.VariableDeclarationList, ts.SyntaxKind.VariableDeclaration]);
                        if (!list) {
                            throw new Error('Unsupported variable declaration');
                        }
                        const listPrefix = list.getSourceFile().text
                            .substring(list.getStart(), list.declarations[0].getStart());
                        span.modification.prefix = 'declare ' + listPrefix + span.modification.prefix;
                        span.modification.suffix = ';';
                    }
                    break;
                case ts.SyntaxKind.Identifier:
                    if (parentSpan) {
                        switch (parentSpan.kind) {
                            case ts.SyntaxKind.ExpressionWithTypeArguments:
                            case ts.SyntaxKind.TypeReference:
                            case ts.SyntaxKind.ClassDeclaration:
                            case ts.SyntaxKind.InterfaceDeclaration:
                            case ts.SyntaxKind.EnumDeclaration:
                            case ts.SyntaxKind.TypeAliasDeclaration:
                            case ts.SyntaxKind.ModuleDeclaration:// (namespaces are a type of module declaration)
                                {
                                    const symbol = this._typeChecker.getSymbolAtLocation(span.node);
                                    if (!symbol) {
                                        throw new Error('Symbol not found');
                                    }
                                    const referencedEntry = this._getEntryForSymbol(symbol);
                                    if (referencedEntry) {
                                        if (!referencedEntry.uniqueName) {
                                            // This should never happen
                                            throw new Error('referencedEntry.uniqueName is undefined');
                                        }
                                        span.modification.prefix = referencedEntry.uniqueName;
                                        // span.modification.prefix += '/**/';
                                    }
                                }
                                break;
                        }
                    }
                    break;
            }
        });
    }
    /**
     * Ensures a unique name for each item in the package typings file.
     */
    _makeUniqueNames() {
        const usedNames = new Set();
        for (const entry of this._entries) {
            let suffix = 1;
            entry.uniqueName = entry.localName;
            while (usedNames.has(entry.uniqueName)) {
                entry.uniqueName = entry.localName + '_' + ++suffix;
            }
            usedNames.add(entry.uniqueName);
        }
    }
    /**
     * Looks up the corresponding Entry for the requested symbol.
     */
    _getEntryForSymbol(symbol) {
        const followAliasesResult = PackageTypingsGenerator._followAliases(symbol, this._typeChecker);
        return this._entriesBySymbol.get(followAliasesResult.symbol);
    }
    /**
     * Looks up the corresponding Entry for the requested symbol.  If it doesn't exist,
     * then it tries to create one.
     */
    _fetchEntryForSymbol(symbol) {
        const followAliasesResult = PackageTypingsGenerator._followAliases(symbol, this._typeChecker);
        if (followAliasesResult.isAmbient) {
            return undefined; // we don't care about ambient definitions
        }
        const followedSymbol = followAliasesResult.symbol;
        if (followedSymbol.flags & (ts.SymbolFlags.TypeParameter | ts.SymbolFlags.TypeLiteral)) {
            return undefined;
        }
        let entry = this._entriesBySymbol.get(followedSymbol);
        if (entry) {
            return entry;
        }
        entry = new Entry({
            localName: symbol.name,
            followedSymbol: followedSymbol,
            importPackagePath: followAliasesResult.importPackagePath,
            importPackageExportName: followAliasesResult.importPackageExportName
        });
        this._entries.push(entry);
        this._entriesBySymbol.set(followedSymbol, entry);
        for (const declaration of followedSymbol.declarations || []) {
            this._collectTypes(declaration);
        }
        return entry;
    }
    _collectTypes(node) {
        switch (node.kind) {
            case ts.SyntaxKind.Block:
                // Don't traverse into code
                return;
            case ts.SyntaxKind.TypeReference: // general type references
            case ts.SyntaxKind.ExpressionWithTypeArguments:// special case for e.g. the "extends" keyword
                {
                    // Sometimes the type reference will involve multiple identifiers, e.g. "a.b.C".
                    // In this case, we only need to worry about importing the first identifier,
                    // so do a depth-first search for it:
                    const symbolNode = PackageTypingsGenerator._findFirstChildNode(node, ts.SyntaxKind.Identifier);
                    if (!symbolNode) {
                        break;
                    }
                    const symbol = this._typeChecker.getSymbolAtLocation(symbolNode);
                    if (!symbol) {
                        throw new Error('Symbol not found for identifier: ' + symbolNode.getText());
                    }
                    this._fetchEntryForSymbol(symbol);
                }
                break; // keep recursing
        }
        for (const child of node.getChildren() || []) {
            this._collectTypes(child);
        }
    }
}
exports.default = PackageTypingsGenerator;

//# sourceMappingURL=PackageTypingsGenerator.js.map
